@@ -50,7 +50,7 @@ void do_deletion(undo_type action)
 			old_amount = number_of_chunks_in(openfile->current);
 #endif
 		/* Move the remainder of the line "in", over the current character. */
-		charmove(&openfile->current->data[openfile->current_x],
+		memmove(&openfile->current->data[openfile->current_x],
 					&openfile->current->data[openfile->current_x + charlen],
 					line_len - charlen + 1);
 #ifndef NANO_TINY
@@ -182,7 +182,9 @@ void chop_word(bool forward)
 	openfile->current_x = is_current_x;
 
 	/* Now kill the marked region and a word is gone. */
-	cut_text();
+	add_undo(CUT);
+	do_snip(FALSE, TRUE, FALSE, FALSE);
+	update_undo(CUT);
 
 	/* Discard the cut word and restore the cutbuffer. */
 	free_lines(cutbuffer);
@@ -268,7 +270,7 @@ void cut_to_eof(void)
  * If until_eof is TRUE, move all text from the current cursor
  * position to the end of the file into the cutbuffer.  If append
  * is TRUE (when zapping), always append the cut to the cutbuffer. */
-void do_cut_text(bool copying, bool marked, bool until_eof, bool append)
+void do_snip(bool copying, bool marked, bool until_eof, bool append)
 {
 #ifndef NANO_TINY
 	linestruct *was_bottom = NULL;
@@ -383,13 +385,14 @@ void cut_text(void)
 		add_undo(CUT);
 	}
 
-	do_cut_text(FALSE, openfile->mark != NULL, FALSE, FALSE);
+	do_snip(FALSE, openfile->mark != NULL, FALSE, FALSE);
 
 	update_undo(CUT);
 #else
 	if (is_cuttable(FALSE))
-		do_cut_text(FALSE, FALSE, FALSE, FALSE);
+		do_snip(FALSE, FALSE, FALSE, FALSE);
 #endif
+	wipe_statusbar();
 }
 
 #ifndef NANO_TINY
@@ -406,11 +409,7 @@ void copy_text(void)
 	ssize_t is_current_lineno = openfile->current->lineno;
 	size_t is_current_x = openfile->current_x;
 
-	/* If there is nothing to copy, don't even try. */
-	if (openfile->current->next == NULL && openfile->current->data[0] == '\0')
-		return;
-
-	do_cut_text(TRUE, mark_is_set, FALSE, FALSE);
+	do_snip(TRUE, mark_is_set, FALSE, FALSE);
 
 	/* If the mark was set, restore the viewport and cursor position. */
 	if (mark_is_set) {
@@ -435,8 +434,9 @@ void cut_till_eof(void)
 	}
 
 	add_undo(CUT_TO_EOF);
-	do_cut_text(FALSE, FALSE, TRUE, FALSE);
+	do_snip(FALSE, FALSE, TRUE, FALSE);
 	update_undo(CUT_TO_EOF);
+	wipe_statusbar();
 }
 
 /* Erase text (current line or marked region), sending it into oblivion. */
@@ -456,9 +456,10 @@ void zap_text(void)
 	/* Use the cutbuffer from the ZAP undo item, so the cut can be undone. */
 	cutbuffer = openfile->current_undo->cutbuffer;
 
-	do_cut_text(FALSE, openfile->mark != NULL, FALSE, TRUE);
+	do_snip(FALSE, openfile->mark != NULL, FALSE, TRUE);
 
 	update_undo(ZAP);
+	wipe_statusbar();
 
 	cutbuffer = was_cutbuffer;
 }
@@ -500,5 +501,6 @@ void paste_text(void)
 	openfile->placewewant = xplustabs();
 
 	set_modified();
+	wipe_statusbar();
 	refresh_needed = TRUE;
 }
