@@ -59,11 +59,9 @@
 #define ISSET(flag) ((FLAGS(flag) & FLAGMASK(flag)) != 0)
 #define TOGGLE(flag) FLAGS(flag) ^= FLAGMASK(flag)
 
-/* Macros for character allocation and more. */
-#define charalloc(howmuch) (char *)nmalloc((howmuch) * sizeof(char))
-#define charealloc(ptr, howmuch) (char *)nrealloc(ptr, (howmuch) * sizeof(char))
-#define charmove(dest, src, n) memmove(dest, src, (n) * sizeof(char))
-#define charset(dest, src, n) memset(dest, src, (n) * sizeof(char))
+/* Macros for allocation of character strings. */
+#define charalloc(howmuch) (char *)nmalloc(howmuch)
+#define charealloc(ptr, howmuch) (char *)nrealloc(ptr, howmuch)
 
 /* In UTF-8 a character is at most six bytes long. */
 #ifdef ENABLE_UTF8
@@ -113,14 +111,11 @@
 #define N_(string) gettext_noop(string)
 		/* Mark a string that will be sent to gettext() later. */
 
-#include <stddef.h>
-#include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <dirent.h>
 #include <regex.h>
 #include <signal.h>
-#include <assert.h>
+#include <stdlib.h>
+#include <sys/stat.h>
 
 /* If we aren't using an ncurses with mouse support, exclude any
  * mouse routines, as they are useless then. */
@@ -147,10 +142,12 @@
 #define BAD_COLOR -2
 #endif
 
+#define STANDARD_INPUT  0
+
 /* Enumeration types. */
 typedef enum {
 	NIX_FILE, DOS_FILE, MAC_FILE
-} file_format;
+} format_type;
 
 typedef enum {
 	HUSH, NOTICE, MILD, ALERT
@@ -188,21 +185,14 @@ typedef struct colortype {
 		/* This syntax's foreground color. */
 	short bg;
 		/* This syntax's background color. */
-	int pairnum;
-		/* The color pair number used for this foreground color and
-		 * background color. */
+	short pairnum;
+		/* The pair number for this foreground/background color combination. */
 	int attributes;
 		/* Pair number and brightness composed into ready-to-use attributes. */
-	int rex_flags;
-		/* The regex compilation flags (with or without REG_ICASE). */
-	char *start_regex;
-		/* The start (or all) of the regex string. */
 	regex_t *start;
-		/* The compiled start (or all) of the regex string. */
-	char *end_regex;
-		/* The end (if any) of the regex string. */
+		/* The compiled regular expression for 'start=', or the only one. */
 	regex_t *end;
-		/* The compiled end (if any) of the regex string. */
+		/* The compiled regular expression for 'end=', if any. */
 	struct colortype *next;
 		/* Next set of colors. */
 	int id;
@@ -216,9 +206,26 @@ typedef struct regexlisttype {
 		/* The next regex. */
 } regexlisttype;
 
+typedef struct augmentstruct {
+	char *filename;
+		/* The file where the syntax is extended. */
+	ssize_t lineno;
+		/* The number of the line of the extendsyntax command. */
+	char *data;
+		/* The text of the line. */
+	struct augmentstruct *next;
+		/* Next node. */
+} augmentstruct;
+
 typedef struct syntaxtype {
 	char *name;
 		/* The name of this syntax. */
+	char *filename;
+		/* File where the syntax is defined, or NULL if not an included file. */
+	size_t lineno;
+		/* The line number where the 'syntax' command was found. */
+	struct augmentstruct *augmentations;
+		/* List of extendsyntax commands to apply when loaded. */
 	regexlisttype *extensions;
 		/* The list of extensions that this syntax applies to. */
 	regexlisttype *headers;
@@ -227,6 +234,8 @@ typedef struct syntaxtype {
 		/* The list of libmagic results that this syntax applies to. */
 	char *linter;
 		/* The command with which to lint this type of file. */
+	char *tab;
+		/* What the Tab key should produce; NULL for default behavior. */
 #ifdef ENABLE_COMMENT
 	char *comment;
 		/* The line comment prefix (and postfix) for this type of file. */
@@ -285,64 +294,44 @@ typedef struct linestruct {
 #endif
 } linestruct;
 
-typedef struct partition {
-	linestruct *filetop;
-		/* The top line of this portion of the file. */
-	linestruct *top_prev;
-		/* The line before the top line of this portion of the file. */
-	char *top_data;
-		/* The text before the beginning of the top line of this portion
-		 * of the file. */
-	linestruct *filebot;
-		/* The bottom line of this portion of the file. */
-	linestruct *bot_next;
-		/* The line after the bottom line of this portion of the
-		 * file. */
-	char *bot_data;
-		/* The text after the end of the bottom line of this portion of
-		 * the file. */
-} partition;
-
 #ifndef NANO_TINY
-typedef struct undo_group {
+typedef struct groupstruct {
 	ssize_t top_line;
 		/* First line of group. */
 	ssize_t bottom_line;
 		/* Last line of group. */
 	char **indentations;
 		/* String data used to restore the affected lines; one per line. */
-	struct undo_group *next;
-} undo_group;
+	struct groupstruct *next;
+		/* The next group, if any. */
+} groupstruct;
 
-typedef struct undo {
-	ssize_t lineno;
+typedef struct undostruct {
 	undo_type type;
-		/* What type of undo this was. */
+		/* The operation type that this undo item is for. */
+	int xflags;
+		/* Some flag data to mark certain corner cases. */
+	ssize_t lineno;
+		/* The line number where the operation began or ended. */
 	size_t begin;
-		/* Where did this action begin or end. */
+		/* The x position where the operation began or ended. */
 	char *strdata;
-		/* String type data we will use for copying the affected line back. */
+		/* String data to help restore the affected line. */
 	size_t wassize;
 		/* The file size before the action. */
 	size_t newsize;
 		/* The file size after the action. */
-	int xflags;
-		/* Some flag data we need. */
-	undo_group *grouping;
+	groupstruct *grouping;
 		/* Undo info specific to groups of lines. */
-
-	/* Cut-specific stuff we need. */
 	linestruct *cutbuffer;
-		/* Copy of the cutbuffer. */
-	linestruct *cutbottom;
-		/* Copy of cutbottom. */
+		/* A copy of the cutbuffer. */
 	ssize_t mark_begin_lineno;
 		/* Mostly the line number of the current line; sometimes something else. */
 	size_t mark_begin_x;
 		/* The x position corresponding to the above line number. */
-	struct undo *next;
+	struct undostruct *next;
 		/* A pointer to the undo item of the preceding action. */
-} undo;
+} undostruct;
 #endif /* !NANO_TINY */
 
 #ifdef ENABLE_HISTORIES
@@ -352,8 +341,9 @@ typedef struct poshiststruct {
 	ssize_t lineno;
 		/* Line number we left off on. */
 	ssize_t xno;
-		/* x position in the file we left off on. */
+		/* The x position in the file we left off on. */
 	struct poshiststruct *next;
+		/* The next item of position history. */
 } poshiststruct;
 #endif
 
@@ -379,10 +369,12 @@ typedef struct openfilestruct {
 		/* The file's x position we would like. */
 	ssize_t current_y;
 		/* The file's y-coordinate position. */
-	bool modified;
-		/* Whether the file has been modified. */
 	struct stat *current_stat;
 		/* The file's current stat information. */
+#ifdef ENABLE_WRAPPING
+	linestruct *spillage_line;
+		/* The line for prepending stuff to during automatic hard-wrapping. */
+#endif
 #ifndef NANO_TINY
 	linestruct *mark;
 		/* The line in the file where the mark is set; NULL if not set. */
@@ -390,19 +382,21 @@ typedef struct openfilestruct {
 		/* The mark's x position in the above line. */
 	mark_type kind_of_mark;
 		/* Whether it is a soft (with Shift) or a hard mark. */
-	file_format fmt;
+	format_type fmt;
 		/* The file's format -- Unix or DOS or Mac or mixed. */
-	undo *undotop;
+	char *lock_filename;
+		/* The path of the lockfile, if we created one. */
+	undostruct *undotop;
 		/* The top of the undo list. */
-	undo *current_undo;
+	undostruct *current_undo;
 		/* The current (i.e. next) level of undo. */
-	undo *last_saved;
+	undostruct *last_saved;
 		/* The undo item at which the file was last saved. */
 	undo_type last_action;
 		/* The type of the last action the user performed. */
-	char *lock_filename;
-		/* The path of the lockfile, if we created one. */
 #endif
+	bool modified;
+		/* Whether the file has been modified. */
 #ifdef ENABLE_COLOR
 	syntaxtype *syntax;
 		/* The  syntax struct for this file, if any. */
@@ -426,17 +420,17 @@ typedef struct rcoption {
 } rcoption;
 #endif
 
-typedef struct sc {
+typedef struct keystruct {
 	const char *keystr;
-		/* The string that describes a keystroke, like "^C" or "M-R". */
+		/* The string that describes the keystroke, like "^C" or "M-R". */
 	bool meta;
 		/* Whether this is a Meta keystroke. */
 	int keycode;
 		/* The integer that, together with meta, identifies the keystroke. */
 	int menus;
-		/* Which menus this applies to. */
+		/* The menus in which this keystroke is bound. */
 	void (*func)(void);
-		/* The function we're going to run. */
+		/* The function to which this keystroke is bound. */
 #ifndef NANO_TINY
 	int toggle;
 		/* If a toggle, what we're toggling. */
@@ -448,11 +442,11 @@ typedef struct sc {
 	char *expansion;
 		/* The string of keycodes to which this shortcut is expanded. */
 #endif
-	struct sc *next;
+	struct keystruct *next;
 		/* Next in the list. */
-} sc;
+} keystruct;
 
-typedef struct subnfunc {
+typedef struct funcstruct {
 	void (*func)(void);
 		/* The actual function to call. */
 	int menus;
@@ -470,9 +464,9 @@ typedef struct subnfunc {
 		/* Is this function allowed when in view mode? */
 	long toggle;
 		/* If this is a toggle, which toggle to affect. */
-	struct subnfunc *next;
+	struct funcstruct *next;
 		/* Next item in the list. */
-} subnfunc;
+} funcstruct;
 
 #ifdef ENABLE_WORDCOMPLETION
 typedef struct completion_word {
@@ -543,7 +537,6 @@ enum
 	AFTER_ENDS,
 	LET_THEM_ZAP,
 	BREAK_LONG_LINES,
-	FINAL_NEWLINE,
 	JUMPY_SCROLLING,
 	EMPTY_LINE
 };
@@ -615,6 +608,9 @@ enum
 #define SHIFT_DELETE 0x45D
 #define SHIFT_TAB 0x45F
 
+/* A special keycode for when <Tab> is pressed while the mark is on. */
+#define INDENT_KEY 0x4F1
+
 #ifdef USE_SLANG
 #ifdef ENABLE_UTF8
 #define KEY_BAD 0xFF  /* Clipped error code. */
@@ -636,9 +632,6 @@ enum
 #define WAS_MARKED_FORWARD    (1<<5)
 #endif /* !NANO_TINY */
 
-/* The maximum number of entries displayed in the main shortcut list. */
-#define MAIN_VISIBLE (((COLS + 40) / 20) * 2)
-
 /* The default number of columns from end of line where wrapping occurs. */
 #define COLUMNS_FROM_EOL 8
 
